/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.              	   |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

package com.ibm.cic.common.core.model;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * A typesafe enumeration of constants representing the kinds
 * of features that may be exposed by an offering.
 */
public class FeatureKind {
	
	/** A constant for an invalid feature kind. */
    public static final FeatureKind NOT_A_FEATURE_KIND
							= new FeatureKind ("NotAValidFeatureKind"); //$NON-NLS-1$

    /** A constant for a feature that is required and visible to the user. */
    public static final FeatureKind REQUIRED_VISIBLE
							= new FeatureKind ("RequiredAndVisible"); //$NON-NLS-1$

    /** A constant for a feature that is required and not visible to the user. */
    public static final FeatureKind REQUIRED_NOT_VISIBLE
							= new FeatureKind ("RequiredAndNotVisible"); //$NON-NLS-1$
    
    /** A constant for a feature that is optional and selected by default. */
    public static final FeatureKind OPTIONAL_SELECTED
							= new FeatureKind ("SelectedByDefault"); //$NON-NLS-1$
    
    /** A constant for a feature that is optional and not selected by default. */
    public static final FeatureKind OPTIONAL_NOT_SELECTED
							= new FeatureKind ("NotSelectedByDefault"); //$NON-NLS-1$

    /**
     * Retrieves the name of the feature kind.
     * 
     * @return Returns the value of the <code>name</code> instance variable.
     */
    public String getName() {
        return name;
    }

    /**
     * Retrieves the enumeration value of the feature kind.
     * 
     * @return Returns the value of the <code>ordinal</code> instance variable.
     */
    public final int getOrdinal() {
        return ordinal;
    }

    /**
     * Retrieves a textual representation of this feature kind.
     * 
     * @return Retutns a textual representation of this feature kind.
     * 
     * @see Object#toString()
     */
    public String toString() {
        return getName();
    }
    
    /**
     * Get a feature kind from its name.
     */
    public static FeatureKind nameToFeatureKind(String s) {
    	Iterator iterator = getFeatureKinds().iterator();
    	while (iterator.hasNext()) {
    		FeatureKind nextFeature = (FeatureKind)iterator.next();
    		if (nextFeature.name.equals(s)) {
    			return nextFeature;
    		}
    	}
        return NOT_A_FEATURE_KIND;
    }

    /**
     * Get a feature kind from it enumeration value of a feature kind to its name.
     */
    public static FeatureKind valueToFeatureKind(int value) {
        if (value <= 0 || value >= FEATURE_KINDS.length) {
            return NOT_A_FEATURE_KIND;
        }
        else {
            return FEATURE_KINDS[value];
        }
    }
    
	/**
     * Retrieves the list of constants for the feature kind enumerated type.
     * 
     * @return Returns the list of constants for this enumerated type.
	 */
	public static List getFeatureKinds() {
        return Collections.unmodifiableList(Arrays.asList(FEATURE_KINDS));
	}

    /**
     * The list of values for the feature kind enumerated type.
     */
    private static final FeatureKind [] FEATURE_KINDS = {NOT_A_FEATURE_KIND,
    												     REQUIRED_VISIBLE,
    							 		   				 REQUIRED_NOT_VISIBLE,
    							 		   				 OPTIONAL_SELECTED,
    							 		   				 OPTIONAL_NOT_SELECTED};
    
    /**
     * An internal ordinal counter for the feature kind enumerated type.
     */
    private static int nextOrdinal = 0;
	
    /**
     * The name of this feature kind enumeration value.
     */
    private final transient String name;

    /**
     * The ordinal for this feature kind enumeration value.
     */
    private final int ordinal;

    /**
     * Constructs a new feature kind with the specified name and ordinal.
     * 
     * @param name The name of the new feature kind.
     */
    protected FeatureKind (String name) {
        super();
        this.name 	 = name;
        this.ordinal = nextOrdinal++;
    }
   
}
